/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JMSConnection.java 4950 2009-08-17 18:49:08Z renaultgu $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.common.jms;

import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.ow2.jasmine.monitoring.eos.common.EventManager;

/**
 * @author poutchjp
 */
public class JMSConnection {

    public static final String INITIAL_CONTEXT_FACTORY = "org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory";
    public static final String TOPIC_FACTORY = "JTCF";
    public static final String TOPIC = "jasmine";

    protected static final int OK = 0;

    protected static final int ERROR = -1;

    protected MessageListener messageListener = null;

    protected InitialContext context = null;

    protected TopicConnectionFactory tcf = null;

    protected TopicConnection connection = null;

    protected TopicSession session = null;

    protected Topic topic = null;

    protected TopicSubscriber subscriber = null;

    private EventManager manager = null;

    public JMSConnection(final EventManager evManager) {
        this.manager = evManager;
    }


    public int connect() {
        try {
            //Properties prop = new Properties();
            //prop.put(Context.INITIAL_CONTEXT_FACTORY, INITIAL_CONTEXT_FACTORY);
            //prop.put(Context.PROVIDER_URL, PROVIDER_URL);

            // context = new InitialContext(prop);

            context = new InitialContext();

            Object o = context.lookup(TOPIC_FACTORY);
            tcf = (TopicConnectionFactory) o;
            topic = (Topic) context.lookup(TOPIC);
            messageListener = new JMSMessageListener(this.manager);

            connection = tcf.createTopicConnection();
            session = connection.createTopicSession(false, Session.CLIENT_ACKNOWLEDGE);

            subscriber = session.createSubscriber(topic);
            subscriber.setMessageListener(messageListener);

            connection.start();

        } catch (NamingException e) {
            // TODO Auto-generated catch block
            // e.printStackTrace();
            return ERROR;
        } catch (JMSException e) {
            // TODO Auto-generated catch block
            // e.printStackTrace();
            return ERROR;
        }
        return OK;
    }


    public int connect(final String topicName,final MessageListener evMessageListener) {
        try {

            context = new InitialContext();

            Object o = context.lookup(TOPIC_FACTORY);
            tcf = (TopicConnectionFactory) o;
            topic = (Topic) context.lookup(topicName);
            messageListener = evMessageListener;

            connection = tcf.createTopicConnection();
            session = connection.createTopicSession(false, Session.CLIENT_ACKNOWLEDGE);

            subscriber = session.createSubscriber(topic);
            subscriber.setMessageListener(messageListener);

            connection.start();

        } catch (NamingException e) {
            // TODO Auto-generated catch block
            // e.printStackTrace();
            return ERROR;
        } catch (JMSException e) {
            // TODO Auto-generated catch block
            // e.printStackTrace();
            return ERROR;
        }
        return OK;
    }


    public int disconnect() {

        try {
            if (this.connection != null) {
                this.session.close();
                this.connection.close();
            }

        } catch (Exception e) {
            return ERROR;
        }

        return OK;

    }

}
